# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nislmigrate',
 'nislmigrate.extensibility',
 'nislmigrate.facades',
 'nislmigrate.logs',
 'nislmigrate.migrators',
 'nislmigrate.utility']

package_data = \
{'': ['*']}

install_requires = \
['argparse>=1.4.0,<2.0.0',
 'cryptography>=35.0.0,<36.0.0',
 'pymongo>=3.12.1,<4.0.0']

entry_points = \
{'console_scripts': ['nislmigrate = nislmigrate.migration_tool:main']}

setup_kwargs = {
    'name': 'nislmigrate',
    'version': '1.1.0',
    'description': 'The tool for migrating SystemLink data.',
    'long_description': "# NI-SystemLink-Migration tool `nislmigrate`\n`nislmigrate` is a command line utility for migration, backup, and restore of supported SystemLink services.\n\n### Prerequisites\n#### 1. SystemLink\n- This tool offically supports migration between SystemLink versions 21.0, 21.1, 21.3, & 21.5. Other versions have not been tested.\n- **We assume the server you are migrating to is clean with no data. Migrating to a server with existing data will result in data loss.**\n- Services that are supported with any caveats and exceptions are detailed in the **Supported Services** section below.\n- This tool assumes a single-box SystemLink installation.\n- This tool must be run on the same machines as the SystemLink installations.\n#### 2. Python\n- This tool requires [Python 3.8](https://www.python.org/downloads/release/python-3811/) to run.\n- The documentation in this repository assumes Python has been added to your [**PATH**](https://datatofish.com/add-python-to-windows-path/).\n### Installation\nThe latest released version of the tool can be installed by running:\n```bash\npip install nislmigrate\n```\n# Usage\n### Backup\n\nTo backup the data for a service listed in the **Supported Services** section run the tool with elevated permissions and use the `capture` option with the corresponding flag for each of the services you want to back up (e.g. `--security`):\n```bash\nnislmigrate capture --security\n```\nThis will backup the data corresponding with each service into the default migration directory (`C:\\Users\\[user]\\Documents\\migration\\`). You can specify a different migration directory using the `--dir [path]` option:\n```bash\nnislmigrate capture --security --dir C:\\custom-backup-location\n```\nTo backup the data for all supported services at once, the `--all` flag can be used instead of listing out each individual service. Using `--all` will require that you include the `--secret` flag for services that require encrypting backed up data for security:\n```bash\nnislmigrate capture --all --secret <password>\n```\n\n### Restore\n\n> :warning: Restoring requires the `--force` flag to explicitly allow overwriting the existing data on the server. Without it, the command will fail.\n\nTo restore the data for a service listed in the **Supported Services** section run the tool with elevated permissions and  use the `restore` option with the corresponding flag for each of the services you want to restore (e.g. `--security`):\n```bash\nnislmigrate restore --security\n```\nThis will restore the data corresponding with each service from the default migration directory (`C:\\Users\\[user]\\Documents\\migration\\`). If your captured data is in a different directory that can be specified with the `--dir [path]` option:\n```bash\nnislmigrate restore --security --dir C:\\custom-backup-location\n```\nTo restore the data for all supported services at once, the `--all` flag can be used instead of listing out each individual service. Using `--all` will require that you include the `--secret` flag for services that require encrypting backed up data for security:\n```bash\nnislmigrate restore --all --secret <password>\n```\n\n### Modify\n\nTo modify entries in the database in-place without doing a restore run the tool with elevated permissions and use the `modify` option. `modify` currently only works to modify the files' service database entries.\n\n#### Updating after moving files\n\nAfter moving the storage location of the files uploaded to the Files service the database will need to be updated to reflect the new location. The `--files-change-file-store-root` argument can be used for this, either as part of a `restore` operation or in-place as part of a `modify` operation. When used with `restore` the old root location is inferred from the database. When used with `modify` the old root location must be specified with the `--files-file-store-root` argument.\n\n> :warning: `modify` will modify the database directly. A backup of the database should be taken before any modifiations using `nislmigrate capture --files --files-metadata-only`\n\nTo modify the files root location after moving the files to a new drive:\n```bash\ncp -r -f C:\\old\\file\\store X:\\new\\file\\store\nnislmigrate modify --files --files-change-file-store-root X:\\new\\file\\store --files-file-store-root C:\\old\\file\\store\n```\n\nTo modify the files root location after moving the files to an S3 bucket:\n```bash\naws s3 sync C:\\old\\file\\store s3://my-systemlink-bucket/my-files\nnislmigrate modify --files --files-change-file-store-root s3://my-systemlink-bucket/my-files --files-file-store-root C:\\old\\file\\store --files-switch-to-forward-slashes\n```\n\n### Migration\n>:warning: Server B must be a clean SystemLink installation, any existing data will be deleted.\n\nTo migrate from one SystemLink server instance (`server A`) to a different instance (`server B`):\n1. Install the migration tool on `server A` and `server B`.\n2. Follow the backup instructions to backup the data from `server A`.\n3. Copy the data produced by the backup of `server A` on `server B`.\n4. **_Warning:_** Ensure `server B` is a clean SystemLink installation, any existing data will be deleted.\n5. Follow the restore instructions to restore the backed up data onto `server B`.\n\n# Development\nSee `CONTRIBUTING.MD` for detailed instructions on developing, testing, and releasing the tool.\n\n# Supported Services\nThe services that can be migrated with this utility along with short descriptions can be listed by running:\n```bash\nnislmigrate capture -h\n```\n\nMost services require migrating the `--security` service at the same time for the migration to be successful, and some services have additional dependencies which are listed in the table below.  \n\n| **Supported Service**                     | **Argument Flag** | **Also requires migrating** | **Additional Notes**                                                                                                                                                                                                                                                                                                                                                                             |\n|---------------------------------|-------------------|-----------------------------|--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|\n| Security                        | `--security`      |                             |                                                                                                                                                                                                                                                                                                                                                                                                  |\n| User Data                       | `--userdata`      | `--security`                |                                                                                                                                                                                                                                                                                                                                                                                                  |\n| Notifications                   | `--notification`  | `--security`                |                                                                                                                                                                                                                                                                                                                                                                                                  |\n| File Ingestion                  | `--files`         | `--security`                | - Must migrate file to the same storage location on the new System Link server.<br>- To capture/restore only the database but not the files themselves, use `--files --files-metadata-only`. This could be useful if, for example, files are stored on a file server with separate backup.<br>- If files are stored in Amazon Simple Storage Service (S3), use `--files --files-metadata-only`.<br>- If the file store path is different on the server you are restoring to, use the `--files-change-file-store-root [NEW_ROOT]` flag to update the metadata of all files to point to the new root during a restore operation.<br>- If you have uploaded your local files to S3 and need to update the file path metadata, use `--files-change-file-store-root [S3://<bucket-name>/<folder-path-if-applicable>]` along with `--files-switch-to-forward-slashes`.  |\n| Repository                      | `--repo`          | `--security`                | - Feeds may require additional updates if servers used for migration have different domain names                                                                                                                                                                                                                                                                                                 |\n| Dashboards and Web Applications | `--dashboards`    | `--security`                |                                                                                                                                                                                                                                                                                                                                                                                                  |\n| System States                   | `--systemstates`  | `--security`                | - Feeds may require additional updates if servers used for migration have different domain names<br>- Cannot be migrated between 2020R1 and 2020R2 servers                                                                                                                                                                                                                                       |\n| Tag Ingestion and Tag History   | `--tags`          | `--security`                |                                                                                                                                                                                                                                                                                                                                                                                                  |\n| Tag Alarm Rules                 | `--tagrule`       | `--security`<br>`--notification` |                                                                                                                                                                                                                                                                                                                                                                                                  |\n| Alarm Instances                 | `--alarms`        | `--security`<br>`--notification` | - Cannot be migrated between 2020R1 and 2020R2 servers                                                                                                                                                                                                                                                                                                                                           |\n| Asset Alarm Rules               | `--assetrule`     | `--security`<br>`--notification` |                                                                                                                                                                                                                                                                                                                                                                                                  |\n| Asset Management                | `--assets`        | `--security`<br>`--files`<br>`--tags`        |                                                                                                                                                                                                                                                                                                                                                                                                  |\n| Test Monitor                    | `--tests`         | `--security`<br>`--file`         |                                                                                                                                                                                                                                                                                                                                                                                                  |\n| Systems                         | `--systems`       | `--security`<br>`--tags`<br>`--file`  | - _WARNING:_ Captured systems data contains encrypted secret information and should not be copied to a publicly accessible location.<br>- To capture/restore systems, a secret must be provided using the `--secret <SECRET>` command line flag. Captured systems data will require the same secret to be provided as was provided during capture in order to be able to decrypt sensitive data. |\n\nThere are plans to support the following services in the future:\n- OPC UA Client: `--opc`\n- TDM `--tdm`\n- Cloud Connector `--cloud`\n",
    'author': 'cnunnall',
    'author_email': 'christian.nunnally@ni.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.8,<4.0.0',
}


setup(**setup_kwargs)
